/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#include "acc_thread.h"
#include "ble_host_thread.h"
#include "acc.h"
#include "config.h"
#include "util.h"
#include "pedometer.h"

static acc_unit_t data[ACC_SET_COUNT];

void acc_thread_entry(void);
void acc_spi_callback(spi_callback_args_t * p_args);
static void process_acc_data (acc_unit_t *data, int count);

/* Accelerometer Thread entry function */
void acc_thread_entry(void)
{
    uint32_t events;

    // make sure all chip selects are driven high
    g_ioport_on_ioport.pinWrite(ACC_CS_PIN, IOPORT_LEVEL_HIGH);
    g_ioport_on_ioport.pinWrite(GYRO_CS_PIN, IOPORT_LEVEL_HIGH);
    g_ioport_on_ioport.pinWrite(MAG_CS_PIN, IOPORT_LEVEL_HIGH);

    // Wait for BLE ready event
    tx_event_flags_get(&g_ble_ready_event, 0x01, TX_OR, &events, TX_WAIT_FOREVER);

    acc_start();

    while (1)
    {
        // wait for accelerometer interrupt
        tx_semaphore_get(&acc_int, 100);

        if (acc_int_service())
        {
            // data is ready, so...
            // 1. retrieve it
            acc_set_get (data, nelem(data));
            // 2. process it
            process_acc_data (data, nelem(data));
        }
    }
}

static void process_acc_data (acc_unit_t *pdata, int count)
{
    pedometer((xyz_count_t *)pdata, count);
}

/*
 * Call back function for accelerometer SPI driver
 */
void acc_spi_callback(spi_callback_args_t * p_args)
{
    (void)p_args;
    tx_semaphore_ceiling_put(&g_acc_semaphore, 1);
}

/*
 * Accelerometer Interrupt Handler
 */
void accel_irq_callback(external_irq_callback_args_t *p_args)
{
    (void) ((p_args));
    // FIFO fill interrupt
    tx_semaphore_put(&acc_int);
}
