/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * pedometer.h
 *
 *  Created on: Sep 18, 2015
 *      Author: xaplos
 ****************************************************************************************
 */

#ifndef PEDOMETER_H_
#define PEDOMETER_H_

#include <stdint.h>
#include <stdbool.h>
#include "config.h"

// Data from accelerometer captured at  62.5 Hz so Tsam = 16ms
#define STEP_MIN_TICK_COUNT (10)         // running; ~160 ms
#define STEP_MAX_TICK_COUNT (125)        // 2 seconds

// Minimum steps to detect before considering walking / running
#define STEP_MIN_FOR_ACTIVITY           2

#define CALORIES_BURNED_PER_MILE        90.0f
#define STEPS_PER_MILE                  2100.0f
#define STEPS_UNTIL_UPDATE              1 //10
#define STEPS_GOAL                      1000          // 1000 goal represented in %

#define CM_PER_M                        100.0f
#define M_PER_MILE                      1609.34f
#define CM_PER_MILE                     (CM_PER_M * M_PER_MILE)

typedef struct
{
    int16_t x;
    int16_t y;
    int16_t z;
}xyz_count_t;

typedef struct
{
    uint32_t tick;          // tick associated with the SAMPLE RATE in the accelerometer
    uint32_t tick_last;     // last tick when a step was detected
    uint32_t count;         // total number of strides
    uint32_t total;         // double of count (?)
    uint8_t cadence;        // repetitions per minute
    uint8_t possible_steps; // possible steps detected
    bool stopped;           // whether it has been detected subject stopped
}step_info_t;

typedef struct
{
    uint16_t calories;
    uint16_t distance;      // in feet so can roughly handle 12.3 miles
    uint16_t activity;
    uint16_t steps;
    uint8_t cadence;
}activity_event_t;

typedef struct
{
    int32_t level_avg;                      // running average
    float calories;
    float distance;
    uint16_t activity;
    // for intermediate purposes
    int32_t level;
    uint32_t squared;
    xyz_count_t count;
    step_info_t step;
}activity_info_t;

void pedometer (xyz_count_t *data, int count);

#endif /* PEDOMETER_H_ */
