/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#ifndef __HUMIDITY_H__
#define __HUMIDITY_H__


/**
 * @brief Pressure sensor return codes.
 */
typedef enum
{
	HUMIDITY_OK = 0,		    /**< no error; success */
	HUMIDITY_WRITE_ERROR,		/**< error writing data to humidity sensor */
	HUMIDITY_READ_ERROR,		/**< error reading data from humidity sensor */
	HUMIDITY_BUSY_ERROR,		/**< error humidity sensor already in use */
	HUMIDITY_ACK_ERROR,		    /**< error acknowledge failure */
	HUMIDITY_TIMEOUT_ERROR,	    /**< error timeout */
	HUMIDITY_INT_ERROR			/**< internal humidity sensor error */
}
HumidityErrorCode;

// there is only one address available for this sensor
#define SHT2x_ADDRESS               (0x40) // 0b100000x

#define SHT2x_RESOLUTION_RH12_T14   0x00    // 0xxxxxx0 DEFAULT
#define SHT2x_RESOLUTION_RH8_T12    0x01    // 0xxxxxx1
#define SHT2x_RESOLUTION_RH10_T13   0x80    // 1xxxxxx0
#define SHT2x_RESOLUTION_RH11_T11   0x81    // 1xxxxxx1
#define SHT2x_STATUS_ABOVE_225      0x00    // x0xxxxxx
#define SHT2x_STATUS_BELOW_225      0x40    // x1xxxxxx
#define SHT2x_ENABLE_ONCHIP_HEATER  0x04    // xxxxx1xx DEFAULT 0
/* OTP Reload is a safety feature and loads the entire OTP settings to the register, 
  with the exception of the heater bit, before every measurement. This feature is 
  disabled per default and is not recommended for use. Please use Soft Reset instead � 
  it contains OTP Reload */
#define SHT2x_DISABLE_OTP_RELOAD    0x02    // xxxxxx1x

#define SHT2x_CMD_UNKNOWN   0

//
// Registers available on the SHT2x
//
// There are two different operation modes to communicate with the sensor: 
// Hold Master mode or No Hold Master mode. In the first case the SCL line is blocked 
// (controlled by sensor) during measurement process while in the latter case the SCL 
// line remains open for other communication while the sensor is processing the measurement. 
// No hold master mode allows for processing other I2C communication tasks on a bus while 
// the sensor is measuring.
#define SHT2x_READ_TEMP_HM  0xE3    // 11100011 
#define SHT2x_READ_RH_HM    0xE5    // 11100101
#define SHT2x_READ_TEMP_NHM 0xF3    // 11110011 
#define SHT2x_READ_RH_NHM   0xF5    // 11110101

#define SHT2x_READ_TEMP     SHT2x_READ_TEMP_NHM
#define SHT2x_READ_RH       SHT2x_READ_RH_NHM
#define SHT2x_WRITE_UREG    0xE6    // 11100110
#define SHT2x_READ_UREG     0xE7    // 11100111
#define SHT2x_SOFT_RESET    0xFE    // 11111110

#define SHT2x_RESERVED_MASK 0x38    // xx111xxx this are reserved bits and should not be
                                    // changed from their power up values

#define SHT2x_RESET_TIMEOUT (15 * 1000)   // 15ms

#define SHT2x_MAX_SCAN      3       // how many times to try to read the data requested
                                    // times out at (SHT2x_MAX_SCAN * SHT2x_SCAN_PERIOD)

#define SHT2x_INVALID_VALUE (-1)

/***************************************************************************/
/**
 *	@brief Initialize humidity sensor.
 *
 ***************************************************************************/
HumidityErrorCode humidity_init(void);

/***************************************************************************/
/**
 *	@brief Trigger humidity read.
 *
 ***************************************************************************/
HumidityErrorCode humidity_trigger(void);

/***************************************************************************/
/**
 *	@brief Read humidity
 *
 *	@param pressure - output humidity reading
 *
 ***************************************************************************/
HumidityErrorCode humidity_read_humidity(float *humidity);

#endif /* __HUMIDITY_H__ */
