/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#ifndef __PRESSURE_H__
#define __PRESSURE_H__

/**
 * @brief Pressure sensor return codes.
 */
typedef enum
{
	PRESSURE_OK = 0,		    /**< no error; success */
	PRESSURE_WRITE_ERROR,		/**< error writing data to pressure sensor */
	PRESSURE_READ_ERROR,		/**< error reading data from pressure sensor */
	PRESSURE_BUSY_ERROR,		/**< error pressure sensor already in use */
	PRESSURE_ACK_ERROR,		    /**< error acknowledge failure */
	PRESSURE_TIMEOUT_ERROR,	    /**< error timeout */
	PRESSURE_INT_ERROR			/**< internal pressure sensor error */
}
PressureErrorCode;

/***************************************************************************/
/**
 *	@brief Initialize pressure sensor.
 *
 ***************************************************************************/
PressureErrorCode pressure_init(void);

/***************************************************************************/
/**
 *	@brief Read pressure
 *
 *	@param pressure - output pressure reading
 *
 ***************************************************************************/
PressureErrorCode pressure_read_pressusre(float *pressure);

/***************************************************************************/
/**
 *	@brief Read temperature
 *
 *	@param temp - output temperature reading
 *
 ***************************************************************************/
PressureErrorCode pressure_read_temp(float *temp);

/***************************************************************************/
/**
 *	@brief Read pressure & temperature
 *
 *  @param pressusre - output pressure reading
 *	@param temp - output temperature reading
 *
 ***************************************************************************/
PressureErrorCode pressure_read_pressure_and_temp(float *pressure, float *temp);

#endif /* __PRESSURE_H__ */
