#include "bsp_api.h"
#include "tx_api.h"
#include "hal_data.h"
#include "x_print.h"
#include "main_thread.h"

#include <stdio.h>
#include <stdarg.h>

/**
 * Is the debugger enabled?
 *
 * @return 1 if enabled, 0 if not
 */
static int x_print_usable(void)
{
    return (CoreDebug->DHCSR & CoreDebug_DHCSR_C_DEBUGEN_Msk) == CoreDebug_DHCSR_C_DEBUGEN_Msk;
}

/**
 * Setup the standard library to use the JTAG for stdout
 */
static int x_print_init(void)
{
    static int x_print_configured = 0;
    int result = 0;

    /* Why is this not in a header? */
    extern void initialise_monitor_handles(void);

    /* Avoid a startup race */
    tx_mutex_get(&g_x_print_lock, TX_WAIT_FOREVER);

    if (x_print_usable())
    {
        result = 1;

        if (!x_print_configured)
        {
            initialise_monitor_handles();
            x_print_configured = 1;
        }
    }

    tx_mutex_put(&g_x_print_lock);

    return result;
}

void x_printf_file_line(const char *filename, int line, const char *format, ...)
{
    va_list v;
    int usable = x_print_init();

    if (usable)
    {
        const TX_THREAD *thread = tx_thread_identify();
        const char *thread_name = "NULL";

        if (thread != NULL && thread->tx_thread_name != NULL)
        {
            thread_name = thread->tx_thread_name;
        }

        tx_mutex_get(&g_x_print_lock, TX_WAIT_FOREVER);
        fprintf(stdout, "[%s] %s:%d ", thread_name, filename, line);
        va_start(v, format);
        vfprintf(stdout, format, v);
        va_end(v);
        fprintf(stdout, "\n");
        tx_mutex_put(&g_x_print_lock);
    }
}

ssp_err_t x_printf_flush(void)
{
	return SSP_SUCCESS;
}

void x_abort_break(void)
{
    /* A place to put a breakpoint */
    while (1)
    {
        __asm__ __volatile__("bkpt 0x00");
    }
}

