package com.xaplos.hcsolution;

import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.content.Intent;

import java.util.HashSet;

public class ConnectActivity extends AppCompatActivity implements BLEManager.BLEManagerListener, BLEManager.BLEManagerNotifyListener {
    private static final String TAG = "ConnectActivity";
    private final BLEManager mBle = BLEManager.getInstance();
    private TextView mConnectedTextView;
    private Button mConnectButton;
    private boolean mConnected = false;
    HashSet<String> mTargets = new HashSet<>();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        Log.d(TAG, "onCreate");
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_connect);
        getSupportActionBar().hide();

        // Names of boards we support
        mTargets.add("Renesas HC Demo");

        mConnectButton = (Button)findViewById(R.id.button_connect);
        mConnectButton.setOnClickListener(mClickListener);

        mConnectedTextView = (TextView)findViewById(R.id.text_view_connected);
        mBle.setListener(this);
        mBle.addNotifyListener(this);
    }

    @Override
    protected void onDestroy() {
        Log.d(TAG, "onDestroy");
        super.onDestroy();
    }

    @Override
    protected void onStart() {
        Log.d(TAG, "onStart");
        super.onStart();
    }

    @Override
    protected void onStop() {
        Log.d(TAG, "onStop");
        super.onStop();
    }

    @Override
    protected void onPause() {
        Log.d(TAG, "onPause");
        super.onPause();
    }

    @Override
    protected void onResume() {
        Log.d(TAG, "onResume");
        super.onResume();
    }

    @Override
    protected void onRestart() {
        Log.d(TAG, "onRestart");
        super.onRestart();
    }

    private View.OnClickListener mClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View view) {
            Log.d(TAG, "click");

            if (!mConnected) {
                mBle.startScan();
            }
            else {
                mBle.disconnect();
            }
        }
    };

    @Override
    public void onDiagnosticMessage(String message) {
        mConnectedTextView.setText(message);
    }

    @Override
    public void onDeviceDiscovered(Bundle device) {
        for (String key : device.keySet()) {
            Log.d(TAG, key + ": " + device.get(key));
        }

        String name = device.getString("name");

        if (name == null) {
            Log.e(TAG, "no device name");
        }
        else if (mTargets.contains(name)) {
            Log.d(TAG, "found, request connect: " + name);
            mBle.connect(device);
        }
    }

    @Override
    public void onDeviceConnected() {
        mConnected = true;
        mConnectButton.setText("Disconnect");
    }

    @Override
    public void onDeviceDisconnected() {
        mConnected = false;
        mConnectButton.setText("Connect");
        // notify everyone about the disconnection
        Intent intent = new Intent();
        intent.setAction("com.xaplos.BleDisconnect");
        sendBroadcast(intent);
    }

    @Override
    public void onNotification(Bundle notification) {
        for (String key : notification.keySet()) {
       //     Log.d(TAG, key + ": " + notification.get(key));
        }
    }
}
