package com.xaplos.hcsolution;

import android.bluetooth.BluetoothGattCharacteristic;
import android.os.Bundle;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Random;
import java.util.UUID;

/*

typedef struct healthcare_data_t
{
    uint16_t spo2;
    uint16_t heart_rate;
    uint16_t systolic;
    uint16_t diastolic;
    uint16_t pulse_rate;
    uint16_t glucose;
    uint16_t bpm_state;
    uint16_t bgm_state;
} healthcare_data;

*/


public class HealthcareDemoCharacteristic extends BLECharacteristic {
    @SuppressWarnings("unused")
    private static final String TAG = "HealthcareDemoCharacteristic";

    // UUID for reporting characteristic of on-board sensor value
    private static final UUID uuidDemoSensor = UUID.fromString("02000000-0000-0000-0000-0000000000C0");

    @Override
    public String getCharacteristicName() {
        return "Healthcare";
    }

    @Override
    public UUID getCharacteristicUuid() {
        return uuidDemoSensor;
    }

    @Override
    public Bundle parseCharacteristicValue(BluetoothGattCharacteristic characteristic) {
        Bundle results = new Bundle();
        ByteBuffer buffer = ByteBuffer.wrap(characteristic.getValue());
        buffer.order(ByteOrder.LITTLE_ENDIAN);

        short raw = buffer.getShort();
        results.putInt("SpO2", raw & 0xffff);
        raw = buffer.getShort();
        results.putInt("BPM", raw & 0xffff);
        raw = buffer.getShort();
        results.putInt("Systolic", raw & 0xffff);
        raw = buffer.getShort();
        results.putInt("Diastolic", raw & 0xffff);
        raw = buffer.getShort();
        results.putInt("PulseRate", raw & 0xffff);
        raw = buffer.getShort();
        results.putInt("Glucose", raw & 0xffff);
        raw = buffer.getShort();
        results.putInt("BpmStatus", raw & 0xffff);
        raw = buffer.getShort();
        results.putInt("BgmStatus", raw & 0xffff);

        return results;
    }

    @Override
    public Bundle emulateCharacteristic() {
        Bundle results = new Bundle();
        Random random = new Random();
        results.putFloat("Sensor", random.nextInt(0xffff));

        return results;
    }
}
