/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * led_pulse_ssp.c
 *
 *  Created on: Dec 16, 2015
 *      Author: alejandro
 ****************************************************************************************
 */

#include "led_pulse.h"
#include "led_pulse_ssp.h"
#include "config.h"

static timer_on_gpt_cfg_t gpt_pulse_red =
{
     .gtioca.output_enabled = true, .gtioca.stop_level = GPT_PIN_LEVEL_LOW,
     .gtiocb.output_enabled = false, .gtiocb.stop_level = GPT_PIN_LEVEL_LOW,
};

static timer_on_gpt_cfg_t gpt_pulse_ired =
{
     .gtioca.output_enabled = false, .gtioca.stop_level = GPT_PIN_LEVEL_LOW,
     .gtiocb.output_enabled = true, .gtiocb.stop_level = GPT_PIN_LEVEL_LOW,
};

static pulse_context_t pulse_context[PULSE_LED_COUNT] =
{
    [PULSE_LED_RED] =
    {
        .timer = &g_timer_1,
        .gpt = &gpt_pulse_red,
        .pin_enable = PULSE_R_PIN,
        0
    },
    [PULSE_LED_IRED] =
    {
        .timer = &g_timer_1,
        .gpt = &gpt_pulse_ired,
        .pin_enable = PULSE_IR_PIN,
        1
    },
};

void hal_pulse_X_drive (pulse_led_t led, bool enable)
{
    pulse_context_t *c = &pulse_context[led];
    const ioport_api_t *api = g_ioport.p_api;

    api->pinWrite (c->pin_enable, enable ? PULSE_ENABLE_LEVEL : PULSE_DISABLE_LEVEL);
}

void hal_pulse_X_open (pulse_led_t led)
{
    pulse_context_t *c = &pulse_context[led];
    const timer_instance_t *t = c->timer;
    timer_ctrl_t *ctrl = t->p_ctrl;

    if (c->open)
    {
        // do nothing ... already open
    }
    else
    {
        c->cfg= *t->p_cfg;    // copy default configuration

        // guarantee regardless of what is there certain configuration
        c->cfg.autostart = false;
        c->cfg.mode = TIMER_MODE_PWM;
        c->cfg.unit = TIMER_UNIT_FREQUENCY_HZ;
        c->cfg.p_extend = c->gpt;
        c->cfg.p_context = &c->cfg;
        //
        ssp_err_t err = t->p_api->open (ctrl, &c->cfg);

        if (err)
        {
            // TODO:
            // handle error condition ....
        }
        else
        {
            c->open = true;
        }
    }
}

void hal_pulse_X_close (pulse_led_t led)
{
    pulse_context_t *c = &pulse_context[led];
    const timer_instance_t *t = c->timer;
    timer_ctrl_t *ctrl = t->p_ctrl;

    t->p_api->close (ctrl);
    c->open = false;
    c->freq = 0;
    c->duty = 0;
}

void hal_pulse_X_stop(pulse_led_t led)
{
    pulse_context_t *c = &pulse_context[led];
    const timer_instance_t *t = c->timer;
    timer_ctrl_t *ctrl = t->p_ctrl;

    t->p_api->stop (ctrl);
}

void hal_pulse_X_modify(pulse_led_t led, uint32_t freq, int duty)
{
    pulse_context_t *c = &pulse_context[led];
    const timer_instance_t *t = c->timer;
    timer_ctrl_t *ctrl = t->p_ctrl;
    timer_api_t const *api = t->p_api;
    ssp_err_t err;

    if (c->freq == freq && c->duty == duty)
    {

    }
    else
    {
        err = api->stop (ctrl);

        if (SSP_SUCCESS == err)
        {
            err = api->periodSet (ctrl, freq, TIMER_UNIT_FREQUENCY_HZ);
        }

        if (SSP_SUCCESS == err)
        {
            err = api->dutyCycleSet (ctrl, duty, TIMER_PWM_UNIT_PERCENT, c->pin);
        }

        if (SSP_SUCCESS == err)
        {
            err = api->start (ctrl);
        }

        if (SSP_SUCCESS == err)
        {
            // remember this
            c->freq = freq;
            c->duty = duty;
        }
    }
}
