/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#include "display_thread.h"
#include "hal_data.h"
#include "main_thread.h"
#include "events.h"
#include "display.h"
#include "driver/led.h"
#include "util.h"
#include "adc.h"

#include "application_dispatcher_thread.h"
#include "common.h"

#define BGM_MIN_READING     69
#define BGM_MAX_READING     200

extern healthcare_data g_healthcare_data;

typedef enum {
    IDLE,
    DISPLAY_IS,
    DISPLAY_BP,
    DISPLAY_BG,

} DISPLAY_STATES;

typedef enum {
    INIT,
    STATE_ONE,
    STATE_TWO,
} DISPLAY_STATE;

typedef struct
{
    TX_EVENT_FLAGS_GROUP *event;
    ULONG actual_flags;
    DISPLAY_STATES state;
    DISPLAY_STATE display_state;
} display_context_t;

display_context_t display_context =
{ .event = &adc_events, .state = DISPLAY_IS, .display_state = STATE_ONE };

void display_thread_entry(void);

static void handle_display_timeout (void);
static void handle_display_IS (void);
static void handle_display_BP (void);
static void handle_display_BG (void);
static void reset_display_timer (void);
//static void stop_display_timer (void);

/* Display Thread entry function */
void display_thread_entry(void)
{
    UINT status;
    display_context_t *c = &display_context;

    status = tx_event_flags_get (c->event, BOARD_INIT_EVENT,
                    TX_OR, &c->actual_flags, TX_WAIT_FOREVER);
    display_str("InS");
    while (1)
    {
        status = tx_event_flags_get (c->event, DISPLAY_MASK,
                TX_OR_CLEAR, &c->actual_flags, TX_WAIT_FOREVER);
        if (status)
                {
                    tx_thread_sleep (1);
                }
                else
                {
                    if (DISPLAY_BG_EVENT & c->actual_flags)
                    {
                        handle_display_BG ();
                    }
                    if (DISPLAY_TIMEOUT_EVENT & c->actual_flags)
                    {
                        handle_display_timeout ();
                    }
                    if (DISPLAY_IS_EVENT & c->actual_flags)
                    {
                        handle_display_IS ();
                    }
                    if (DISPLAY_BP_EVENT & c->actual_flags)
                    {
                        handle_display_BP ();
                    }

                }
    }
    while (1)
    {
        tx_thread_sleep (1);
    }
}


void handle_display_timeout (void)
{
    display_context_t *c = &display_context;
    switch (c->state)
        {
        case DISPLAY_IS:
            c->display_state = (c->display_state == STATE_ONE) ? STATE_TWO : STATE_ONE;
           (c->display_state == STATE_ONE) ? display_str("InS") : display_str("STr");
            break;
        case DISPLAY_BP:
            c->display_state = (c->display_state == STATE_ONE) ? STATE_TWO : STATE_ONE;
           (c->display_state == STATE_ONE) ? display_str("InS") : display_str("bLd");
            break;
        case DISPLAY_BG:

            break;
        default:
            break;
        }

}

static void handle_display_IS (void)
{
    display_context_t *c = &display_context;
    c->state = DISPLAY_IS;
    c->display_state = INIT;
    reset_display_timer ();
}

static void handle_display_BP (void)
{
    display_context_t *c = &display_context;
    c->state = DISPLAY_BP;
    c->display_state = INIT;
    reset_display_timer ();
}

static void handle_display_BG (void)
{
    int bg_val;
    UINT err;

    display_context_t *c = &display_context;
    c->state = DISPLAY_BG;
    c->display_state = INIT;
    display_str("---");
    //stop_display_timer ();
    err = tx_queue_receive(&bg_queue,&bg_val,500);
    if (TX_SUCCESS != err)
    {
        display_str("FA ");
        led_red(STATUS_LED);
        // update data for BLE reporting
        tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);
        g_healthcare_data.bgm_state = BGM_FAILED_STATE;
        tx_mutex_put(&g_data_mutex);
    }
    else
    {
        // sanity check on reading
        // fail on anything less than 70 and over 200
        if ((BGM_MIN_READING < bg_val) && (BGM_MAX_READING > bg_val))
        {
            // successful reading
            display_int (bg_val);
            // update data for BLE reporting
            tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);
            g_healthcare_data.glucose = (uint16_t)bg_val;
            tx_mutex_put(&g_data_mutex);
            led_green(STATUS_LED);
        }
        else
        {
            display_str("FA ");
            led_red(STATUS_LED);
            // update data for BLE reporting
            tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);
            g_healthcare_data.bgm_state = BGM_FAILED_STATE;
            tx_mutex_put(&g_data_mutex);
        }

    }


}

static void reset_display_timer (void)
{
    display_context_t *c = &display_context;
    display_timer.p_api->close(display_timer.p_ctrl);
    display_timer.p_api->open(display_timer.p_ctrl,display_timer.p_cfg);
    tx_event_flags_set (c->event, DISPLAY_TIMEOUT_EVENT, TX_OR);
}
/*
static void stop_display_timer (void)
{
    display_timer.p_api->close(display_timer.p_ctrl);
}
*/
void display_timer_callback(timer_callback_args_t * p_args)
{
    UNUSED(p_args);

    display_context_t *c = &display_context;
    tx_event_flags_set (c->event, DISPLAY_TIMEOUT_EVENT, TX_OR);
}
