/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * x_print.h
 *
 *  Created on: Mar 23, 2016
 *      Author: alejandro
 ****************************************************************************************
 */

#ifndef X_PRINT_H_
#define X_PRINT_H_

#include <stdio.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdbool.h>
#include "sf_comms_api.h"

//#define X_PRINTF_WAIT (10*TX_TIMER_TICKS_PER_SECOND)

#ifndef CDC_WRITE_TIMEOUT_SEC
#define CDC_WRITE_TIMEOUT_SEC   5
#endif

extern void initialise_monitor_handles (void);

/**
 * @brief x_print_njtag initializes x_printf logger without JTAG support
 * @note ... THIS or x_print_init needs to be called before any other API
 *        can be used
 * @param none
 * @return none
 */
void x_print_njtag (void);

/**
 * @brief x_print_init initializes x_printf logger with/without JTAG support
 * @note ... THIS or x_print_njtag needs to be called before any other API
 *        can be used
 * @param jtag_use : whether JTAG logging is enabled (not recommended) or not
 * @return none
 */
void x_print_init (bool jtag_use);

/***
 * @brief queues printf like formated string from application (thread) context
 *      NOTE: Do not call from ISR context
 * @param format, printf like format and parameters
 * @return none
 */
void x_printf(const char *format, ...);

/***
 * @brief flushes any pending x_printf calls out through the communication pathway
 * @return none
 */
ssp_err_t x_printf_flush(void);

/***
 * @brief queues printf like formated string from ISR context
 *      NOTE: Is the only allowed way to log from ISR context
 * @param format, printf like format and parameters
 * @return none
 */
void x_printf_isr(const char *format, ...);

/***
 * @brief Any user specific initialization required
 * @param none
 * @return none
 */
void x_printf_user_init (void);

// lazy macro to call flusher also
#define x_printff( ...)     \
    x_printf(__VA_ARGS__);  \
    x_printf_flush ();

/***
 * @brief queues raw bytes for transmission (thread) context
 *      NOTE: Do not call from ISR context
 * @param data pointer to the data to be transmitted
 * @param count how many bytes to be transmitted
 * @return none
 */
void x_print_raw(uint8_t *data, int count);

/***
 * @brief flushes any pending x_print_raw/x_printf calls out through the communication pathway
 * @return none
 */
void x_print_raw_flush(void);

#endif /* X_PRINT_H_ */
