/**
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 *
 * @file		rscip_uart.c
 *
 * @brief	RSCIP UART transport module functions.
 *
 * Copyright(C) 2013-2014 Renesas Electronics Corporation
 *
 * $Rev: 2156 $
 *
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rscip.h"
#include "rscip_uart.h"
#include "rscip_cntl.h"
#include "ble_app_thread.h"
#include <stdbool.h>

/*
 * DEFINES
 ****************************************************************************************
 */


/*
 * TYPE DEFINITIONS
 ****************************************************************************************
 */


/*
 * GLOBAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */
#define TX_BUFF 4

RSCIP_Uart_t		uart[RSCIP_WINDOW_SIZE + 1];
RSCIP_Uart_t		*uart_rp;
RSCIP_Uart_t		*uart_wp;

RSCIP_Uart_Ctr_t	uart_ctr;

uint8_t		uart_rx_buf;
uint8_t		uart_tx_buf[TX_BUFF][(RSCIP_MAX_PAYLOAD_SIZE + RSCIP_PACKET_HEADER_SIZE + RSCIP_CHECKSUM_DATA_SIZE + 1) * 1]; 
uint8_t     u_wp=0;
static	RSCIP_UART_STATE	rscip_tx_state;
static	RSCIP_UART_STATE	rscip_rx_state;
static	uint8_t		esc_flag;
static	uint8_t		check_sum;
static	uint16_t	packet_length;

static RSCIP_UART_STATE	last_rscip_rx_state;

static  uint8_t *ble_rx_ptr;
static  uint16_t ble_rx_count;
static  uint16_t ble_rx_size;
/*
 * FUNCTION DEFINITIONS
 ****************************************************************************************
 */

bool serial_read(uint8_t *bufptr, uint16_t size )
{
    ble_rx_ptr = bufptr;
    ble_rx_size = size;
    ble_rx_count = 0;
    return TRUE;
}

/******************************************************************************************/
/**
 *  @brief UART initializing process
 *
 *  An UART is initialized for this function.@n
 *  
 *  @attention
 *  
 *  @b Contents of management:
 *    -# Initialization of a UART driver.
 *    -# Initialization of a variable.
 *    -# UART reception start.
 *
 *  @param[in]
 *
 *  @retval
 */
/******************************************************************************************/

bool RSCIP_Uart_Init(void)
{
	bool		Ret;
	uint8_t		i;
    ssp_err_t err;

//	Ret = serial_init();


    err = g_ble_uart.p_api->open(g_ble_uart.p_ctrl, g_ble_uart.p_cfg);
    Ret = (err == SSP_SUCCESS);
	if ( Ret ) {
	
		for (i = 0; i < (RSCIP_WINDOW_SIZE + 1); i++) {
			uart[i].rx_flag = FALSE;
		}
	
		uart_ctr.rp = 0;
		uart_ctr.wp = 0;
	
		tx_done.app  = FALSE;
		tx_done.uart = FALSE;
	
	    //	initialize tx state
	    rscip_tx_state = RSCIP_UART_STATE_TX_IDLE;

		//start uart reception
	    Ret = RSCIP_Uart_Read_Start();

        last_rscip_rx_state = RSCIP_UART_STATE_RX_RESET;
        rscip_rx_state      = RSCIP_UART_STATE_RX_RESET;
	}
	return( Ret );
}


/******************************************************************************************/
/**
 *  @brief UART request to send processing
 *
 *  This function does a request-to-send to a UART driver.@n
 *  
 *  @attention
 *  
 *  @b Contents of management:
 *
 *  @param[in]		*headerptr		header buffer address
 *  @param[in]		*bufptr			packet buffer address
 *  @param[in]		checksum		checksum data
 *  @param[in]		size			The number of send data.
 *
 *  @retval
 */
/******************************************************************************************/

bool RSCIP_Uart_Write(uint8_t *headerptr, uint8_t *bufptr, uint8_t checksum, uint16_t size)
{
	uint16_t	i;
	uint16_t	tx_data_count;
	ssp_err_t err;

	if (RSCIP_UART_STATE_TX_IDLE != rscip_tx_state) {
		return false;
	}

	tx_data_count = 1;
	uart_tx_buf[u_wp][0] = END_CODE;

	if (NULL != headerptr) {
		for (i = 0; i < RSCIP_PACKET_HEADER_SIZE; i++, tx_data_count++) {
			switch (headerptr[i]) {
				case END_CODE:
					uart_tx_buf[u_wp][tx_data_count++] = ESC_CODE;
					uart_tx_buf[u_wp][tx_data_count]   = ESC_END_CODE;
					break;
				case ESC_CODE:
					uart_tx_buf[u_wp][tx_data_count++] = ESC_CODE;
					uart_tx_buf[u_wp][tx_data_count]   = ESC_ESC_CODE;
					break;
				default:
					uart_tx_buf[u_wp][tx_data_count]   = headerptr[i];
					break;
			}
		}

		for (i = 0; i < size; i++, tx_data_count++) {
			switch (bufptr[i]) {
				case END_CODE:
					uart_tx_buf[u_wp][tx_data_count++] = ESC_CODE;
					uart_tx_buf[u_wp][tx_data_count]   = ESC_END_CODE;
					break;
				case ESC_CODE:
					uart_tx_buf[u_wp][tx_data_count++] = ESC_CODE;
					uart_tx_buf[u_wp][tx_data_count]   = ESC_ESC_CODE;
					break;
				default:
					uart_tx_buf[u_wp][tx_data_count]   = bufptr[i];
					break;
			}
		}

		if (RSCIP_INTEGRITY_TYPE_ON == ((rscip.config & RSCIP_INTEGRITY_TYPE_MASK) >> RSCIP_INTEGRITY_TYPE_SIFT)) {
			switch (checksum)
			{
				case END_CODE:
					uart_tx_buf[u_wp][tx_data_count++] = ESC_CODE;
					uart_tx_buf[u_wp][tx_data_count++] = ESC_END_CODE;
					break;
				case ESC_CODE:
					uart_tx_buf[u_wp][tx_data_count++] = ESC_CODE;
					uart_tx_buf[u_wp][tx_data_count++] = ESC_ESC_CODE;
					break;
				default:
					uart_tx_buf[u_wp][tx_data_count++] = checksum;
					break;
			}
		}
	}
	else {
		for (i = 0, tx_data_count = 1; i < size; i++, tx_data_count++) {
			switch (bufptr[i]) {
				case END_CODE:
					uart_tx_buf[u_wp][tx_data_count++] = ESC_CODE;
					uart_tx_buf[u_wp][tx_data_count]   = ESC_END_CODE;
					break;
				case ESC_CODE:
					uart_tx_buf[u_wp][tx_data_count++] = ESC_CODE;
					uart_tx_buf[u_wp][tx_data_count]   = ESC_ESC_CODE;
					break;
				default:
					uart_tx_buf[u_wp][tx_data_count]   = bufptr[i];
					break;
			}
		}
	}

	uart_tx_buf[u_wp][tx_data_count++] = END_CODE;

	rscip_tx_state = RSCIP_UART_STATE_TX_ONGOING;
//	serial_write(uart_tx_buf, tx_data_count);
	err = g_ble_uart.p_api->write(g_ble_uart.p_ctrl,uart_tx_buf[u_wp],tx_data_count);

    u_wp++;
	
    if(u_wp%TX_BUFF)
   	{
   	  ;
   	}else {
   	  u_wp =0;
   	}
		
	return(SSP_SUCCESS == err);
}


/******************************************************************************************/
/**
 *  @brief UART reception request processing
 *
 *  This function does a reception request to a UART driver.@n
 *  
 *  @attention
 *  
 *  @b Contents of management:
 *
 *  @param[in]
 *
 *  @retval
 */
/******************************************************************************************/

bool RSCIP_Uart_Read_Start(void)
{
    //	initialize rx state
    rscip_rx_state = RSCIP_UART_STATE_RX_END_SEARCH;
	esc_flag = FALSE; 

	//	Set the UART environment to message type 1 byte reception
	return serial_read(&uart_rx_buf, 1);
}


/******************************************************************************************/
/**
 *  @brief UART transmission completion processing
 *
 *  This function does transmission completion processing of a UART driver.@n
 *  
 *  @attention
 *  
 *  @b Contents of management:
 *
 *  @param[in]
 *
 *  @retval	
 */
/******************************************************************************************/

void RSCIP_Uart_Tx_Done(void)
{

	rscip_tx_state = RSCIP_UART_STATE_TX_IDLE;

    // Defer the freeing of resources to ensure that it is done in background
	tx_done.uart = true;
    ke_evt_set( EVT_RSCIP_TX_DONE_BIT );
}


/******************************************************************************************/
/**
 *  @brief UART reception completion processing
 *
 *  This function does reception completion processing of a UART driver.@n
 *  
 *  @attention
 *  
 *  @b Contents of management:
 *
 *  @param[in]
 *
 *  @retval
 */
/******************************************************************************************/

void RSCIP_Uart_Rx_Done(void)
{
    last_rscip_rx_state = rscip_rx_state;
	switch (rscip_rx_state) {

		case RSCIP_UART_STATE_RX_END_SEARCH:
		case RSCIP_UART_STATE_RX_RESET:

			if (END_CODE == uart_rx_buf) {
				rscip_rx_state = RSCIP_UART_STATE_RX_START;
			}
			serial_read(&uart_rx_buf, 1);

			break;

		case RSCIP_UART_STATE_RX_START:

			if (END_CODE == uart_rx_buf) {
				rscip_rx_state = RSCIP_UART_STATE_RX_HEADER0;
				serial_read(&uart_rx_buf, 1);

			}
			else {
				uart_wp = &uart[uart_ctr.wp];
				if (true == uart_wp->rx_flag) {
					RSCIP_Uart_Read_Start();
					return;
				}

				if (true == esc_flag) {
					esc_flag = false;
					if (ESC_END_CODE == uart_rx_buf) {
						uart_rx_buf = END_CODE;
					}
					else {
						uart_rx_buf = ESC_CODE;
					}
				}
				else {
					if (ESC_CODE == uart_rx_buf) {
						esc_flag = true;
						serial_read(&uart_rx_buf, 1);
						return;
					}
				}

				uart_wp->rx_header_buf[0] = uart_rx_buf;
				check_sum = uart_rx_buf;
				rscip_rx_state = RSCIP_UART_STATE_RX_HEADER1;
				serial_read(&uart_rx_buf, 1);
			}
			break;

		case RSCIP_UART_STATE_RX_HEADER0:

			uart_wp = &uart[uart_ctr.wp];
			if (true == uart_wp->rx_flag) {
				RSCIP_Uart_Read_Start();
				return;
			}

			if (true == esc_flag) {
				esc_flag = false;
				if (ESC_END_CODE == uart_rx_buf) {
					uart_rx_buf = END_CODE;
				}
				else {
					uart_rx_buf = ESC_CODE;
				}
			}
			else {
				if (ESC_CODE == uart_rx_buf) {
					esc_flag = true;
					serial_read(&uart_rx_buf, 1);
					return;
				}
			}

			uart_wp->rx_header_buf[0] = uart_rx_buf;
			check_sum = uart_rx_buf;
			rscip_rx_state = RSCIP_UART_STATE_RX_HEADER1;
			serial_read(&uart_rx_buf, 1);
			break;

		case RSCIP_UART_STATE_RX_HEADER1:

			if (true == esc_flag) {
				esc_flag = false;
				if (ESC_END_CODE == uart_rx_buf) {
					uart_rx_buf = END_CODE;
				}
				else {
					uart_rx_buf = ESC_CODE;
				}
			}
			else {
				if (ESC_CODE == uart_rx_buf) {
					esc_flag = true;
					serial_read(&uart_rx_buf, 1);
					return;
				}
			}

			uart_wp->rx_header_buf[1] = uart_rx_buf;
			check_sum = (uint8_t)(check_sum + uart_rx_buf);
			packet_length = (uint16_t)(uart_rx_buf & RSCIP_PAYLOAD_LENGTH_MASK);
			rscip_rx_state = RSCIP_UART_STATE_RX_HEADER2;
			serial_read(&uart_rx_buf, 1);
			break;

		case RSCIP_UART_STATE_RX_HEADER2:

			if (true == esc_flag) {
				esc_flag = false;
				if (ESC_END_CODE == uart_rx_buf) {
					uart_rx_buf = END_CODE;
				}
				else {
					uart_rx_buf = ESC_CODE;
				}
			}
			else {
				if (ESC_CODE == uart_rx_buf) {
					esc_flag = true;
					serial_read(&uart_rx_buf, 1);
					return;
				}
			}

			uart_wp->rx_header_buf[2] = uart_rx_buf;
			check_sum = (uint8_t)(check_sum + uart_rx_buf);
			packet_length = (uint16_t)((packet_length << RSCIP_PAYLOAD_LENGTH_SIFT) + uart_rx_buf);
			rscip_rx_state = RSCIP_UART_STATE_RX_HEADER3;
			serial_read(&uart_rx_buf, 1);
			break;

		case RSCIP_UART_STATE_RX_HEADER3:

			if (true == esc_flag) {
				esc_flag = false;
				if (ESC_END_CODE == uart_rx_buf) {
					uart_rx_buf = END_CODE;
				}
				else {
					uart_rx_buf = ESC_CODE;
				}
			}
			else {
				if (ESC_CODE == uart_rx_buf) {
					esc_flag = true;
					serial_read(&uart_rx_buf, 1);
					return;
				}
			}

			uart_wp->rx_header_buf[3] = uart_rx_buf;
			check_sum = (uint8_t)(check_sum + uart_rx_buf);

			if (0x00 == check_sum) {
				if (0 == packet_length) {
					rscip_rx_state = RSCIP_UART_STATE_RX_END;
					serial_read(&uart_rx_buf, 1);
				}
				else {
					if (RSCIP_INTEGRITY_CHECK == (uart_wp->rx_header_buf[0] & RSCIP_INTEGRITY_CHECK_MASK)) {
						packet_length = (uint16_t)(packet_length + RSCIP_CHECKSUM_DATA_SIZE);
					}
					rscip_rx_state = RSCIP_UART_STATE_RX_PAYLOAD;
					serial_read(uart_wp->rx_buf, ( uint16_t ) (packet_length + 1));
				}
			}
			else {	//	Header Check Sum Error
				RSCIP_Uart_Read_Start();
			}

			break;

		case RSCIP_UART_STATE_RX_PAYLOAD:

			if (END_CODE == uart_wp->rx_buf[packet_length]) {

				uart_wp->rx_length = packet_length;
				uart_wp->rx_flag = true;

				uart_ctr.wp++;
				if ((RSCIP_WINDOW_SIZE + 1) <= uart_ctr.wp) {
				    // Lokeshif ((RSCIP_WINDOW_SIZE + 1) <= uart_ctr.wp) {
					uart_ctr.wp = 0;
				}

				//	initialize rx state
				rscip_rx_state = RSCIP_UART_STATE_RX_START;
				esc_flag = false;
				//Set the UART environment to message type 1 byte reception
				serial_read(&uart_rx_buf, 1);
				// Defer the freeing of resources to ensure that it is done in background
				ke_evt_set( EVT_RSCIP_RX_DONE_BIT );
			}
			else {
				packet_length++;
				serial_read(&uart_wp->rx_buf[packet_length], 1);
			}
			break;


		case RSCIP_UART_STATE_RX_END:

			if (END_CODE == uart_rx_buf) {

				uart_wp->rx_length = packet_length;
				uart_wp->rx_flag = true;

				uart_ctr.wp++;
				if ((RSCIP_WINDOW_SIZE + 1) <= uart_ctr.wp) {
					uart_ctr.wp = 0;
				}

				//	initialize rx state
				rscip_rx_state = RSCIP_UART_STATE_RX_START;
				esc_flag = false;
				//Set the UART environment to message type 1 byte reception
				serial_read(&uart_rx_buf, 1);

				// Defer the freeing of resources to ensure that it is done in background
				ke_evt_set( EVT_RSCIP_RX_DONE_BIT );
			}
			else {
				RSCIP_Uart_Read_Start();
			}
			break;

		default:
			//	error
			RSCIP_Uart_Read_Start();
			break;
	}
}


/******************************************************************************************/
/**
 *  @brief UART reception error handling
 *
 *  This function does an error handling of a UART driver.@n
 *  
 *  @attention
 *  
 *  @b Contents of management:
 *    -# A reception request is performed to a UART driver.
 *
 *  @param[in]
 *
 *  @retval	
 */
/******************************************************************************************/

void RSCIP_Uart_Rx_Error(void)
{
	RSCIP_Uart_Read_Start();
}

/******************************************************************************************/
/**
 *  @brief State check of a RSCIP driver
 *
 *  This function judges whether a RSCIP is in the state of after acquisition packet of header.@n
 *  
 *  @param[in]
 *
 *  @retval			true:		in the state of after acquisition packet of header
 *					false:		in the other state
 */
/******************************************************************************************/

bool RSCIP_Uart_Rx_FirstByte(void)
{
	if((RSCIP_UART_STATE_RX_START      == last_rscip_rx_state) &&
	   (RSCIP_UART_STATE_RX_END_SEARCH !=      rscip_rx_state))
	{
		return true;
	}
	return false;
}
/******************************************************************************************/
/**
 *  @brief State check of a RSCIP driver
 *
 *  This function judges whether a RSCIP is in the packet complete state.@n
 *  
 *  @param[in]
 *
 *  @retval			true:		in the packet complete state
 *					false:		in the other state
 */
/******************************************************************************************/

bool RSCIP_Uart_Rx_Complete(void)
{
	if((RSCIP_UART_STATE_RX_START ==      rscip_rx_state) &&
	   (RSCIP_UART_STATE_RX_START != last_rscip_rx_state) &&
	   (RSCIP_UART_STATE_RX_RESET != last_rscip_rx_state)
	   )
	{
		return true;
	}
	return FALSE;
}

/******************************************************************************************/
/**
 *  @brief State check of a RSCIP driver
 *
 *  This function judges whether a RSCIP screwdriver is in the sleep possible state.@n
 *  
 *  @attention
 *  
 *  @b Contents of management:
 *    -# A transmission/receiving condition is checked.
 *
 *  @param[in]
 *
 *  @retval			true:		sleep OK
 *					false:		sleep NG
 */
/******************************************************************************************/

bool RSCIP_Can_Sleep(void)
{
	if (RSCIP_UART_STATE_TX_IDLE != rscip_tx_state)
	{
		return	FALSE;
	}
	if ((RSCIP_UART_STATE_RX_START      == rscip_rx_state) ||
		(RSCIP_UART_STATE_RX_END_SEARCH == rscip_rx_state) ||
		(RSCIP_UART_STATE_RX_RESET      == rscip_rx_state))
	{
		return	true;
	}
	return	FALSE;
}

void g_ble_uart_callback(uart_callback_args_t * p_args)
{
    switch (p_args->event)
    {
        case UART_EVENT_RX_COMPLETE :  ///< Receive complete event
            RSCIP_Uart_Rx_Done();
            break;
        case UART_EVENT_RX_CHAR     :  ///< Character received
            *ble_rx_ptr++ = (uint8_t)(p_args->data);    /* Store received character */
            ble_rx_count++;
            if (ble_rx_count == ble_rx_size)
            {
                RSCIP_Uart_Rx_Done();
            }
            break;
        case UART_EVENT_TX_COMPLETE :  ///< Transmit complete event
            RSCIP_Uart_Tx_Done();
            break;
        case UART_EVENT_ERR_PARITY  :  ///< Parity error event
        case UART_EVENT_ERR_FRAMING :  ///< Mode fault error event
        case UART_EVENT_BREAK_DETECT : ///< Break detect error event
        case UART_EVENT_ERR_OVERFLOW : ///< FIFO Overflow error event
        case UART_EVENT_ERR_RXBUF_OVERFLOW :  ///< Receive buffer overflow error event
        default :
            RSCIP_Uart_Rx_Error();
            break;

    }
}
