/**
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 *
 * @file    db_handle.h
 *
 * @brief   Definition of Database Handle for Profile
 *
 * Copyright(C) 2013-2015 Renesas Electronics Corporation
 *
 * This file is generated by Bluetooth Developer Studio plugin.
 *     BDS Version    : 1.0.2095.0
 *     Script Version : 1.0.2
 *
 ****************************************************************************************
 */


#ifndef _DB_HANDLE_H_
#define _DB_HANDLE_H_

#include "prf_sel.h"

/**
 * TYPE DEFINES
 ****************************************************************************************
 */

#define HID_BOOT_OTHER                  0u
#define HID_BOOT_KB                     1u
#define HID_BOOT_MO                     2u

#define HIDS_INST1                      HID_BOOT_KB
#define HIDS_INST2                      HID_BOOT_MO

/** Attribute database handles */
enum
{
    /* Generic Access Profile Service*/
    GAP_HDL_PRIM_SVC = 0x0001,
    GAP_HDL_CHAR_DEVNAME,
    GAP_HDL_DEVNAME,
    GAP_HDL_CHAR_ICON,
    GAP_HDL_ICON,
    GAP_HDL_CHAR_SLAVE_PREF_PARAM,
    GAP_HDL_SLAVE_PREF_PARAM,

    /* Generic Attribute Profile Service */
    GATT_HDL_PRIM_SVC,
    GATT_HDL_CHAR_SVC_CHANGED,
    GATT_HDL_SVC_CHANGED,
    GATT_HDL_SVC_CHANGED_IND_CFG,

    #if defined(USE_FW_UPDATE_PROFILE)
    FWUS_HDL_SVC,
    FWUS_HDL_DATA_CNTL_CHAR,
    FWUS_HDL_DATA_CNTL_VAL,
    FWUS_HDL_DATA_CHAR,
    FWUS_HDL_DATA_VAL,
    #endif
    
    #if (USE_LINK_LOSS_SERVICE)
    /* Link Loss Service */
    LLS_HDL_PRIM_SVC,
    LLS_HDL_ALERT_LVL_CHAR,
    LLS_HDL_ALERT_LVL_VAL,
    #endif /* #if (USE_LINK_LOSS_SERVICE) */
    #if (USE_TX_POWER_SERVICE)
    /* TX Power Service */
    TPS_HDL_PRIM_SVC,
    TPS_HDL_TXPW_LVL_CHAR,
    TPS_HDL_TXPW_LVL_VAL,
    #endif /* #if (USE_TX_POWER_SERVICE) */

    #if (USE_IMMEDIATE_ALERT_SERVICE)
    /* Immediate Alert Service */
    IAS_HDL_PRIM_SVC,
    IAS_HDL_ALERT_LVL_CHAR,
    IAS_HDL_ALERT_LVL_VAL,
    #endif /* #if (USE_IMMEDIATE_ALERT_SERVICE) */

    #if (USE_DEV_INFO_SERVICE || USE_DEV_INFO_SERVICE_PNP)
    /* Device information service */
    DIS_HDL_SVC,
    #if (USE_DEV_INFO_SERVICE)
    DIS_HDL_SYS_ID_CHAR,
    DIS_HDL_SYS_ID_VAL,
    DIS_HDL_MODEL_NB_CHAR,
    DIS_HDL_MODEL_NB_VAL,
    DIS_HDL_SERIAL_NB_CHAR,
    DIS_HDL_SERIAL_NB_VAL,
    DIS_HDL_FW_REV_CHAR,
    DIS_HDL_FW_REV_VAL,
    DIS_HDL_HW_REV_CHAR,
    DIS_HDL_HW_REV_VAL,
    DIS_HDL_SW_REV_CHAR,
    DIS_HDL_SW_REV_VAL,
    DIS_HDL_MANUF_NAME_CHAR,
    DIS_HDL_MANUF_NAME_VAL,
    DIS_HDL_IEEE_CERTIF_CHAR,
    DIS_HDL_IEEE_CERTIF_VAL,
    #endif /* #if (USE_DEV_INFO_SERVICE) */
    #if (USE_DEV_INFO_SERVICE_PNP)
    DIS_HDL_PNP_ID_CHAR,
    DIS_HDL_PNP_ID_VAL,
    #endif /* #if (USE_DEV_INFO_SERVICE_PNP) */
    #endif /* #if (USE_DEV_INFO_SERVICE || USE_DEV_INFO_SERVICE_PNP) */

    #if (USE_BATTERY_SERVICE)
    /* Battery Service */
    BAS1_HDL_SVC,
    BAS1_HDL_BATTERY_LVL_CHAR,
    BAS1_HDL_BATTERY_LVL_VAL,
    BAS1_HDL_BATTERY_LVL_CFG,
    BAS1_HDL_CHAR_PRESEN_FMT,
    #ifdef HID_MULTI_INSTANCE
    BAS2_HDL_SVC,
    BAS2_HDL_BATTERY_LVL_CHAR,
    BAS2_HDL_BATTERY_LVL_VAL,
    BAS2_HDL_BATTERY_LVL_CFG,
    BAS2_HDL_CHAR_PRESEN_FMT,
    #endif /* #ifdef HID_MULTI_INSTANCE */
    #endif /* #if (USE_BATTERY_SERVICE) */

    #if (USE_SCAN_PARAMETER_SERVICE)
    /* Scan Parameters Service */
    SCANS_HDL_SVC,
    SCANS_HDL_INTV_WINDOW_CHAR,
    SCANS_HDL_INTV_WINDOW_VAL,
    SCANS_HDL_REFRESH_CHAR,
    SCANS_HDL_REFRESH_VAL,
    SCANS_HDL_REFRESH_CFG,
    #endif /* #if (USE_SCAN_PARAMETER_SERVICE) */

    #ifdef USE_SAMPLE_PROFILE
    /* Sample Custom Service */
	SCS_HDL_SVC_SERVICE1,
	SCS_HDL_SERVICE1_SV1_CHAR1_NTF_CHAR,
	SCS_HDL_SERVICE1_SV1_CHAR1_NTF_VAL,
	SCS_HDL_SERVICE1_SV1_CHAR1_NTF_CFG,
    SCS_HDL_SERVICE1_SV1_CHAR2_NTF_CHAR,
	SCS_HDL_SERVICE1_SV1_CHAR2_NTF_VAL,
	SCS_HDL_SERVICE1_SV1_CHAR2_NTF_CFG,
	SCS_HDL_SERVICE1_SV1_CHAR3_W_CHAR,
	SCS_HDL_SERVICE1_SV1_CHAR3_W_VAL,
    #endif /* #ifdef USE_SAMPLE_PROFILE */

    DB_HDL_MAX
};

#endif /* _DB_HANDLE_H_ */
