/*
  ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * adc.c
 *
 *  Created on: Jul 28, 2016
 *      Author: mitsumono
 ****************************************************************************************
 */

#include "adc.h"
#include "util.h"
#include "driver/x_print.h"
void dummy_adc_data(void);

/*
 * adc_sample_context contains all variables and structs necessary for adc.c functions
 */
adc_sample_context_t adc_sample_context =
{
  .res = &adc_mcu_res,
  .retry_count = 0,
  .timeout_tick = 0,
  .val[ADC_BP_CUFF_VALUE].channel = ADC_REG_CHANNEL_0,
  .val[ADC_BP_CUFF_VALUE].value = 0,
  .val[ADC_BP_OPAMP_VALUE].channel = ADC_REG_CHANNEL_1,
  .val[ADC_BP_OPAMP_VALUE].value = 0,
  .val[ADC_BATT_VALUE].channel = ADC_REG_CHANNEL_4,
  .val[ADC_BATT_VALUE].value = 0,
};

/**
 * @brief pulse_adc_init, refer to header file for details
 */
int adc_notify_init(const adc_sample_context_t *c)
{
    int err;

    err = hal_adc_notify_init (c->res);

    return err;
}

/**
 * @brief pulse_adc_close, refer to header file for details
 */
int adc_notify_close(const adc_sample_context_t *c)
{
    int err;

    err = hal_adc_notify_close (c->res);

    return err;
}

/**
 * @brief pulse_adc_notify, refer to header file for details
 */
int adc_notify(const adc_sample_context_t *c, int event)
{
    int err;

    err = hal_adc_notify (c->res, event);

    return err;
}

/**
 * @brief pulse_adc_init, refer to header file for details
 */
int adc_init(adc_sample_context_t *c)
{
    int err;

    err = adc_notify_init (c);

    if (0 == err)
    {
        err = adc_open (c);
    }

    if (0 == err)
    {
        err = sample_timer_open (c);
    }

    return err;
}

/**
 * @brief adc_close, refer to header file for details
 */
int adc_close(const adc_sample_context_t *c)
{
    int err;

    err = adc_notify_close (c);

    if (0 == err)
    {
        err = sample_timer_close (c);
    }

    if (0 == err)
    {
        err = hal_adc_close (c->res);
    }

    return err;
}

/**
 * @brief sample_adc_open, refer to header file for details
 */
int adc_open(const adc_sample_context_t *c)
{
    int err;

    err = hal_adc_open (c->res);

    return err;
}

/**
 * @brief sample_timer_open, refer to header file for details
 */
int sample_timer_open(const adc_sample_context_t *c)
{
    int err;

    err = hal_sample_timer_open (c->res);

    return err;
}

/**
 * @brief sample_timer_close, refer to header file for details
 */
int sample_timer_close(const adc_sample_context_t *c)
{
    int err;

    err = hal_sample_timer_close (c->res);

    return err;
}

/**
 * @brief adc_convert_start, refer to header file for details
 */
int adc_scan_start(const adc_sample_context_t *c)
{
    int err;

    err = hal_adc_scan_start (c->res);

    return err;
}

int adc_wait_for_event(adc_sample_context_t *c, ULONG *event, results_t *r)
{
    int err;

    err = hal_adc_wait_for_event (c->res, event);

#if defined (USE_DUMMY_DATA)
    *bg_opamp_val = dummy_data[dummy_data_head];
    dummy_data_head++;
    while (dummy_data_head >= DATA_COUNT -1)
    {
        tx_thread_sleep (10);
    }
#else
    if (0 == err)
    {
        err = adc_read_all (c);                                                 //Read all adc channels (can add logic to read individually to save cycles)
    }

    if (0 == err)
    {
        r->bp_adc_cuff_val = adc_sample_context.val[ADC_BP_CUFF_VALUE].value;
        r->bp_adc_opamp_val = adc_sample_context.val[ADC_BP_OPAMP_VALUE].value;
        r->batt_value = adc_sample_context.val[ADC_BATT_VALUE].value;
    }
#endif

    return err;
}

/**
 * @brief pulse_adc_sample_read, refer to header file for details
 */
int adc_read(adc_sample_context_t *c, adc_value_t *val)
{
    int err;

    err = hal_adc_sample_read (c->res, val);

    return err;
}

/**
 * @brief pulse_adc_sample_handle, refer to header file for details
 */
int adc_sample_handle(adc_sample_context_t *c)
{
    int err = SSP_ERR_UNSUPPORTED;
    int event = ADC_EVENT_NONE;

    // at this point something got sampled
        switch (c->mode)
        {
            case ADC_BATT_SAMPLE:
                event = ADC_BATT_EVENT_FLAG;
            break;

            case ADC_BP_BATT_SAMPLE:
                event = ADC_BP_BATT_EVENT_FLAG;
            break;

            case ADC_TEST:
                event = ADC_BP_BATT_EVENT_FLAG;
            break;

            default:
                __asm__ ("nop");
            break;
        }
        // TODO:
        // notify of this value acquired
        err = adc_notify (c, event);

        return err;
}

#define ADC_RES (1 << 14)

float adc_percent(int adc)
{
    float temp;

    temp = (float) adc;
    temp *= 100;
    temp /= ADC_RES;

    return temp;
}

int adc_read_all(adc_sample_context_t *c)
{
    int err = 0;
    for (i = 0; i < ADC_VALUE_COUNT; i++)
    {
        err = adc_read (c, &c->val[i]);
    }
    return err;
}

/**
 * ADC ISR callback
 */
void adc_callback(adc_callback_args_t * p_args)
{
    UNUSED(p_args);
    adc_sample_handle (&adc_sample_context);
}

void test_timer_callback(timer_callback_args_t * p_args)
{
    UNUSED(p_args);
    adc_sample_handle (&adc_sample_context);
}


