/*
  ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * adc_process.h
 *
 *  Created on: Jul 29, 2016
 *      Author: mitsumono
 ****************************************************************************************
 */
#ifndef ADC_PROCESS_H_
#define ADC_PROCESS_H_

#include "adc.h"
#include "config.h"
#include "driver/led.h"
#include "io.h"
#include "filter.h"

#define DEADBAND_HALF 750
#define TWO_THIRDS_BATT FULL_BAT-(FULL_BAT-DEAD_BAT)*1/3
#define TWO_THIRDS_BATT_DEADBAND_UPPER TWO_THIRDS_BATT+DEADBAND_HALF
#define TWO_THIRDS_BATT_DEADBAND_LOWER TWO_THIRDS_BATT-DEADBAND_HALF

#define ONE_THIRD_BATT FULL_BAT-(FULL_BAT-DEAD_BAT)*2/3
#define ONE_THIRD_BATT_DEADBAND_UPPER ONE_THIRD_BATT+DEADBAND_HALF
#define ONE_THIRD_BATT_DEADBAND_LOWER ONE_THIRD_BATT-DEADBAND_HALF

#define TS_BUFF_SIZE 10

#define ONE_SIXTY_MMGH 10500
#define FOURTY_MMGH 3600
#define ZERO_MMGH 1200

#define QUARTER_SECOND 125

#define THIRTY_BPM 1000
#define ONE_HUNDRED_BPM 300

#define LOWER_NOISE_THRESHOLD 0
#define UPPER_NOISE_THRESHOLD 400

#define ALLOWED_PEAK_FAILS 0

/*
 * adc_peak_t contains variables for calculating the peak and elapsed time since the peak
 */
typedef enum
{
    MEASUREMENT_SUCCESS,
    CONTINUE_MEASUREMENT,
    MEASURE_FAILED
} adc_measurement_return;

typedef enum
{
    FIRST_PEAK,
    INCONSISTANT_PEAKS,
    PEAK_PERIOD_OUT_OF_RANGE,
    MISSED_PEAK,
    PEAK_VALID
} peak_error_t;


typedef struct
{
    int previous_filtered_value;
    int variable_threshold;
    int max_threshold;
    int current_peak_pressure;
    long int tracked_bpm;
    long int beats_per_minute;
    int peak_buff_head;
    int peak_buff_start;
    int curr_peak;
    int local_peak;
    int peak_failed_count;
    int missed_peak_count;
    long int peak_timestamps[TS_BUFF_SIZE];
    int peak_pulses[TS_BUFF_SIZE];
    int peak_thresholds[TS_BUFF_SIZE];
    int diastolic_pressures[TS_BUFF_SIZE];
} adc_peak_t;


extern results_t results;
extern adc_peak_t adc_peak;

void adc_process_bp(adc_state_t *state, results_t *r);

adc_measurement_return find_diastolic_pressure (int value, int adc_val, int pressure, long int timestamp, adc_peak_t *adc_p);
adc_measurement_return find_bpm (int value, int adc_val, long int timestamp, adc_peak_t *adc_p);
adc_measurement_return find_systolic_pressure (int pressure, adc_peak_t *adc_p);

int convert_pressure (int adc_pressure);

void set_variable_threshold (int value, adc_peak_t *adc_p);

peak_error_t validate_peak (int peak_period, adc_peak_t *adc_p);

void adc_process_batt(int value);

void adc_process_bp_batt(adc_state_t *state, results_t *r);

int adc_idle_enter (void);
int adc_idle_exit (void);

int bp_batt_adc_init(void);
int bp_batt_adc_close(void);


#endif /* ADC_PROCESS_H_ */
