/*
  ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#include "adc_thread.h"
#include "adc_process.h"
#include "driver/x_print.h"
void adc_thread_entry(void);
static void handle_display_info (results_t *r);
int low_pass;
/*
 * adc_context_t contains all necessary variables for adc_thread_entry
 */
typedef struct
{
    TX_EVENT_FLAGS_GROUP *event;
    TX_MUTEX *mutex;
    TX_QUEUE *queue;
    adc_sample_context_t  *adc_context;
    results_t *results;
} adc_context_t;

adc_context_t adc_top_context =
{ .event = &adc_events,
  .mutex = &adc_mutex,
  .queue = &bp_queue,
  .adc_context = &adc_sample_context,
  .results = &results
};

/* ADC Thread entry function */
void adc_thread_entry(void)
{
    int err = 0;
    ULONG event;                                                                                                        //Contains actual flags
    adc_context_t *c = &adc_top_context;
    results_t *r = c->results;

    err = tx_event_flags_get (c->event, BOARD_INIT_EVENT_ADC,                                                           //Wait for the board to be initialized
                        TX_OR_CLEAR, &event, TX_WAIT_FOREVER);

    while (err==0)
    {
        adc_wait_for_event (c->adc_context, &event, r);                                                                 //Wait for an event dealing with the adc

        if (event & ADC_BATT_EVENT_FLAG)                                                                                //Run if we are only sampling battery level right now
        {
            adc_process_batt (r->batt_value);                                                                           //Process battery adc value
        }

        if (event & ADC_BP_BATT_EVENT_FLAG)                                                                             //Run if we are sampling battery and cuff
        {
            adc_process_bp_batt (&c->adc_context->state, r);                                                            //Process battery and cuff adc values
            handle_display_info (r);                                                                                    //Send results to display_thread_entry if they are valid
        }
        //x_printff("%i\n\r", adc_peak.variable_threshold);                                                             //Print adc output to usb if debugging
        x_printff("%i_%i_%i\r", low_pass, r->bp_value, adc_peak.variable_threshold);                                    //Print adc output to usb if debugging
    }
    led_white (STATUS_LED);                                                                                             //Turn on LED if there is an error

    while (1)
    {
        tx_thread_sleep (1);
    }
}

static void handle_display_info (results_t *r)
{
    adc_context_t *c = &adc_top_context;
    if (r->systolic_pressure >0 && r->beats_per_minute > 0 && r->systolic_pressure > 0 && r->diastolic_pressure > 0)    //Only send if all values have been found
    {
        tx_queue_send(c->queue,&r->beats_per_minute,TX_WAIT_FOREVER);                                                   //Send bpm value to display_thread_entry
        tx_queue_send(c->queue,&r->systolic_pressure,TX_WAIT_FOREVER);                                                  //Send systolic value to display_thread_entry
        tx_queue_send(c->queue,&r->diastolic_pressure,TX_WAIT_FOREVER);                                                 //Send diastolic value to display_thread_entry
        r->beats_per_minute = 0;                                                                                        //Clear results to prevent resending
        r->systolic_pressure = 0;
        r->diastolic_pressure = 0;
    }
}
