/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * d7seg.c
 *
 *  Created on: Dec 9, 2015
 *      Author: alejandro
 ****************************************************************************************
 */

#include "d7seg.h"
#include <stdio.h>
#include <stdbool.h>

/**
 * Forward declaration of some helper routines
 */
static uint8_t d7seg2code (char in);
static const d7seg_lines_t *d7seg_info (int pos);
static void d7seg_display_char (int pos, char ch);
static void d7seg_display_dot (int pos, int v);

static const d7seg_disp_t d7seg_context =
{
    D7SEG_DEF(1),
#if LCD_DIGITS > 1
    D7SEG_DEF(2),
#endif
#if LCD_DIGITS > 2
    D7SEG_DEF(3),
#endif
};

/***
 * @brief Please refer to the header for details
 */
void d7seg_open (void)
{
    hal_d7seg_open ();
}

/***
 * @brief Please refer to the header for details
 */
void d7seg_close (void)
{
    hal_d7seg_close ();
}


/***
 * @brief Please refer to the header for details
 */
void d7seg_display_f (float f, int pres)
{
    char tmp[LCD_DIGITS + LCD_HAS_DOT + 1];
    const char *format = D7SEG_FLOAT_FORMAT;

    switch (pres)
    {
        case 1:
            format = D7SEG_FLOAT_FORMAT1;
            break;

        case 2:
            format = D7SEG_FLOAT_FORMAT2;
            break;
    }
    // format the value into a string
    snprintf (tmp, sizeof (tmp), format, f);
    // write it to the display
    d7seg_display_str (tmp);
}

/***
 * @brief Please refer to the header for details
 */
void d7seg_display_i (int val)
{
    char tmp[LCD_DIGITS + LCD_HAS_DOT + 1];
    const char *format = D7SEG_INT_FORMAT;

    // format the value into a string
    snprintf (tmp, sizeof (tmp), format, val);
    // write it to the display
    d7seg_display_str (tmp);
}

/***
 * @brief Please refer to the header for details
 */
void d7seg_display_str (char *str)
{
    int pos = 0;
    // kill any dots
    for (int i = 0; i < LCD_DIGITS; i++)
    {
        d7seg_display_dot (i, false);
    }
    //
    for (int i = 0; i < LCD_DIGITS + LCD_HAS_DOT && str[i]; i++)
    {
        if ('.' == str[i])
        {
            // adjust char postion for next iteration
            pos--;
            //
            d7seg_display_dot (pos, true);
        }
        else
        {
            d7seg_display_char (pos, str[i]);
        }

        pos++;
    }
}

/**
 * Helper routines go below
 */

static void d7seg_display_char (int pos, char ch)
{
    // this is implementation specific
    int code;
    const d7seg_lines_t *p = d7seg_info (pos);

    // got one?
    if (p)
    {
        uint8_t v;
        // get segments to set/clear
        code = d7seg2code (ch);
        // write them to the LCD
        for (int i = 0; i < 7; i++)
        {
            // on or off ...
            v = code & (1 << i) ? 1 : 0;
            //
            hal_d7seg_write (p->seg[i], v);
        }
    }
}

static void d7seg_display_dot (int pos, int v)
{
    const d7seg_lines_t *p = d7seg_info (pos);

    // got one ?
    if (p && p->dot != D7SEG_INVALID)
    {
        // actually write the segment
        hal_d7seg_write (p->dot, v);
    }
}

static const d7seg_lines_t *d7seg_info (int pos)
{
    const d7seg_lines_t *info = NULL;

    // bound check
    if (pos >= 0 && pos < LCD_DIGITS)
    {
        info = &d7seg_context.pos[pos];
    }

    return info;
}

//
// Encoded letter and number for 7 Segment display
//
static const d7seg_t d7seg_xltor[] =
{
    {'0', D7SEG_A | D7SEG_B | D7SEG_C | D7SEG_D | D7SEG_E | D7SEG_F},
    {'1', D7SEG_B | D7SEG_C},
    {'2', D7SEG_A | D7SEG_B | D7SEG_G | D7SEG_E | D7SEG_D},
    {'3', D7SEG_A | D7SEG_B | D7SEG_G | D7SEG_C | D7SEG_D},
    {'4', D7SEG_F | D7SEG_G | D7SEG_B | D7SEG_C},
    {'5', D7SEG_A | D7SEG_F | D7SEG_G | D7SEG_C | D7SEG_D},
    {'6', D7SEG_A | D7SEG_F | D7SEG_E | D7SEG_D | D7SEG_C | D7SEG_G},
    {'7', D7SEG_A | D7SEG_B | D7SEG_C},
    {'8', D7SEG_A | D7SEG_B | D7SEG_C | D7SEG_D | D7SEG_E | D7SEG_F | D7SEG_G},
    {'9', D7SEG_A | D7SEG_B | D7SEG_C | D7SEG_F | D7SEG_G},
    {'_', D7SEG_D},
    {'-', D7SEG_G},
    // brake between numbers and everything else
    {' ', D7SEG_OFF},
    {'.', D7SEG_DP},
    {'A', D7SEG_A | D7SEG_F | D7SEG_G | D7SEG_B | D7SEG_E | D7SEG_C},
    {'b', D7SEG_F | D7SEG_G | D7SEG_E | D7SEG_C | D7SEG_D},
    {'C', D7SEG_A | D7SEG_F | D7SEG_E | D7SEG_D},
    {'c', D7SEG_D | D7SEG_E | D7SEG_G},
    {'d', D7SEG_B | D7SEG_G | D7SEG_E | D7SEG_C | D7SEG_D},
    {'E', D7SEG_A | D7SEG_F | D7SEG_G | D7SEG_E | D7SEG_D},
    {'e', D7SEG_A | D7SEG_F | D7SEG_G | D7SEG_E | D7SEG_D},
    {'F', D7SEG_A | D7SEG_F | D7SEG_G | D7SEG_E},
    {'f', D7SEG_A | D7SEG_F | D7SEG_G | D7SEG_E},
    {'G', D7SEG_A | D7SEG_F | D7SEG_E | D7SEG_D | D7SEG_C | D7SEG_G},
    {'g', D7SEG_A | D7SEG_F | D7SEG_E | D7SEG_D | D7SEG_C | D7SEG_G},
    {'H', D7SEG_F | D7SEG_G | D7SEG_B | D7SEG_E | D7SEG_C},
    {'h', D7SEG_F | D7SEG_G | D7SEG_E | D7SEG_C},
    {'I', D7SEG_F | D7SEG_E},
    {'i', D7SEG_E},
    {'L', D7SEG_F | D7SEG_E | D7SEG_D},
    {'l', D7SEG_F | D7SEG_E | D7SEG_D},
    {'n', D7SEG_G | D7SEG_E |D7SEG_C},
    {'R', D7SEG_G | D7SEG_E},
    {'r', D7SEG_G | D7SEG_E},
    {'S', D7SEG_A | D7SEG_F | D7SEG_G | D7SEG_C | D7SEG_D},
    {'T', D7SEG_F | D7SEG_G | D7SEG_E | D7SEG_D},
    {'t', D7SEG_F | D7SEG_G | D7SEG_E | D7SEG_D},

    {0, 0},
};

static uint8_t d7seg2code (char in)
{
    uint8_t code = D7SEG_G;   // in case of an error just display '-'

    for (int i = 0; d7seg_xltor[i].ascii; i++)
    {
        if (in == d7seg_xltor[i].ascii)
        {
            code = d7seg_xltor[i].code;
            break;
        }
    }

    return code;
}

