/*
  ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * filter.c
 *
 *  Created on: Aug 22, 2016
 *      Author: mitsumono
 ****************************************************************************************
 */

#include "filter.h"

adc_filter_t adc_filter;

void calculate_running_average (double *ri, double input, double lp);

void filter_adc(int val, int *low_pass, int *filtered_value)
{
    calculate_low_pass (val, &adc_filter.moving_average_one_prev, &adc_filter.moving_average_one_curr, K_ONE);
    calculate_running_average (&adc_filter.ri_one, val, adc_filter.moving_average_one_curr);
    calculate_low_pass (adc_filter.ri_one, &adc_filter.low_pass_one_prev, &adc_filter.low_pass_one_curr, L_ONE);
    calculate_low_pass (adc_filter.low_pass_one_curr, &adc_filter.moving_average_two_prev, &adc_filter.moving_average_two_curr, K_TWO);
    calculate_running_average (&adc_filter.ri_two, adc_filter.low_pass_one_curr, adc_filter.moving_average_two_curr);
    calculate_low_pass (adc_filter.ri_two, &adc_filter.low_pass_two_prev, &adc_filter.low_pass_two_curr, L_ONE);

    *low_pass = (int)adc_filter.moving_average_one_curr;
    *filtered_value = (int)adc_filter.low_pass_two_curr;
}

void calculate_low_pass (double input, double *prev_value, double *curr_value, double k)
{
    *prev_value = *curr_value;
    *curr_value = (1 - k) * *prev_value + k * input;
}

void calculate_running_average (double *ri, double input, double lp)
{
    *ri = input - lp;
}

void update_threshold (int *ri, double input, int *prev_value, double k)
{
    double curr_value;
    curr_value = (1 - k) * (double)*prev_value + k * input;
    *prev_value = (int)curr_value;
    *ri = (int)curr_value;
}

/**
 *  @brief Perform running average
 *
 *  @param kavg - gain factor
 *  @param prev_out - previous avg
 *  @param input - raw input value
 *  @param first_call - flag indicating first time execution
 *
 *  @return final averaged value
 */
int running_avg(double kavg, int prev_out, int input, bool first_call)
{
    int avg_result = 0;

    if (first_call)
    {
        // first time calling function
        avg_result = input;
    }
    else
    {
        prev_out = (int)((kavg * input) + ((1.0 - kavg) * prev_out));
        avg_result = prev_out;
    }

    // return averaged result
    return avg_result;
}
