/*
  ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * io.c
 *
 *  Created on: Aug 14, 2016
 *      Author: mitsumono
 ****************************************************************************************
 */
#include "io.h"
#include "util.h"
ioport_port_pin_t pump_io = IOPORT_PORT_05_PIN_05;
ioport_port_pin_t valve_io = IOPORT_PORT_04_PIN_06;
bool p_on;

/**
 * @brief valve_open calls the function to open the valve
 * @return err if there is an error
 */
int valve_open (void)
{
    int err;
    err = hal_io_high (valve_io);
    return err;
}

/**
 * @brief valve_close calls the function to close the valve
 * @return err if there is an error
 */
int valve_close (void)
{
    int err;
    err = hal_io_low (valve_io);
    return err;
}

/**
 * @brief pwm_start starts the timer on which the pump is controlled
 * @return err if there is an error
 */
int pwm_start (void)
{
#if defined (USE_DUMMY_DATA)
    return 0;
#else
    return pwm_timer.p_api->open(pwm_timer.p_ctrl,pwm_timer.p_cfg);
#endif
}

/**
 * @brief pwm_stop closes the timer on which the pump is controlled amd turns the pump off
 * @return err if there is an error
 */
int pwm_stop (void)
{
    pwm_timer.p_api->close(pwm_timer.p_ctrl);
    tx_thread_sleep (1);    //Pause to make sure last interrupt has been serviced
    return pump_off ();
}

/**
 * @brief pump_on calls the function to turn on the pump
 * @return err if there is an error
 */
int pump_on (void)
{
    return hal_io_low (pump_io);
}

/**
 * @brief pump_off calls the function to turn off the pump
 * @return err if there is an error
 */
int pump_off (void)
{
    return hal_io_high (pump_io);
}

/**
 * @brief pwm_callback is the callback function for the pwm timer controlling the pump
 */
void pwm_callback(timer_callback_args_t * p_args)
{
    UNUSED(p_args);

    p_on = (p_on == true ) ? false : true;
    (p_on == true ) ? pump_on () : pump_off ();
}
