//
//  EmailViewController.m
//  BLE-Demo
//
//  Created by James Roth on 6/10/14.
//  Copyright (c) 2014 Xaplos, Inc. All rights reserved.
//

#import "EmailViewController.h"
#import "AppDelegate.h"

#import <MessageUI/MessageUI.h>
#import <MessageUI/MFMailComposeViewController.h>

@interface EmailViewController ()

@end

@implementation EmailViewController
@synthesize logView;

- (void) returnToStartScreen
{
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"Main" bundle: nil];
    UIViewController * vc = [storyboard instantiateViewControllerWithIdentifier:@"StartScreen"];
    [self presentViewController:vc animated:YES completion:nil];
}

- (IBAction)presentMailer:(id)sender
{
    if (![MFMailComposeViewController canSendMail]) {
        NSString *message = @"This device does not support email. You may need to add an email account.";
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Problem" message:message preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *okButton = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
            // nothing to do
        }];
        [alert addAction:okButton];
        [self presentViewController:alert animated:YES completion:nil];
        
        return;
    }
    
    MFMailComposeViewController *picker = [[MFMailComposeViewController alloc] init];
    picker.mailComposeDelegate = self;

    NSArray *recipients = [NSArray arrayWithObject:@"info@xaplos.com"];
    NSString *logPath = [AppDelegate debugLogPath];
    NSString *contents = [logView text];
    NSData *data = [contents dataUsingEncoding:NSUTF8StringEncoding];

    [picker setSubject:@"BLE Solution Kit Log"];
    [picker setToRecipients:recipients];
    [picker addAttachmentData:data mimeType:@"text/plain" fileName:logPath];
    [picker setMessageBody:@"<b>Please describe the problem here:\n</b>" isHTML:TRUE];
    [self presentViewController:picker animated:YES completion:nil];
}

- (void)mailComposeController:(MFMailComposeViewController *)controller
          didFinishWithResult:(MFMailComposeResult)result
                        error:(NSError *)error
{
    [self dismissViewControllerAnimated:TRUE completion:nil];
    [self returnToStartScreen];
}

- (IBAction)clearLog:(id)sender
{
    [logView setText:@""];
    [AppDelegate redirectNSLogToDocuments:TRUE];
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    NSLog(@"enter settings");
    [super viewDidLoad];
    
    NSString *logPath = [AppDelegate debugLogPath];
    NSError *error;
    NSString *logString = [NSString stringWithContentsOfFile:logPath encoding:NSUTF8StringEncoding error:&error];
    
    if (logString) {
        NSRegularExpression *regex = [NSRegularExpression regularExpressionWithPattern:@"\n[0-9]{4}-[0-9]{2}-[0-9]{2} "
                                                                               options:0
                                                                                 error:&error];

        NSRegularExpression *regex2 = [NSRegularExpression regularExpressionWithPattern:@" BleSolutionKit\\[[0-9a-f:]*\\] "
                                                                               options:0
                                                                                 error:&error];
        // Remove date (not time)
        NSString *modifiedString = [regex stringByReplacingMatchesInString:logString
                                                                   options:0
                                                                     range:NSMakeRange(0, [logString length])
                                                              withTemplate:@"\n"];
        // Remove process name and pid
        NSString *modifiedString2 = [regex2 stringByReplacingMatchesInString:modifiedString
                                                                   options:0
                                                                     range:NSMakeRange(0, [modifiedString length])
                                                              withTemplate:@" "];
        [logView setText:modifiedString2];
    }
    else {
        [logView setText:[error localizedDescription]];
    }
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)cancelEmail:(UIButton *)sender {
    [self returnToStartScreen];
}

@end
