//
//  GlucoseViewController.m
//  BLE Solution
//
//  Created by Mark F Rodriguez on 7/26/16.
//  Copyright © 2016 Mark F Rodriguez. All rights reserved.
//

#import "GlucoseViewController.h"

@interface GlucoseViewController ()

typedef enum
{
    BGM_IDLE_STATE,
    BGM_WAITING_FOR_STRIP_STATE,
    BGM_STRIP_INSERTED_STATE,
    BGM_MEASURE_STATE,
    BGM_FINISH_STATE,
    BGM_FAILED_STATE,
    BGM_MAX_STATES
} bgm_state_t;

@end

@implementation GlucoseViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Tell notification center to inform on "HealthDataReceived" notifications
    // using the bleNotification: selector. By specifying object:nil, informing
    // the notification center we are not interested in who posted the notification.
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(glucoseNotification:)
                                                 name:@"HealthDataReceived"
                                               object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(glucoseDisconnect:)
                                                 name:@"BleDisconnected"
                                               object:nil];
}

- (void) glucoseNotification:(NSNotification *) notification
{
    if ([[notification name] isEqualToString:@"HealthDataReceived"]) {
        NSDictionary *values = [NSDictionary dictionaryWithDictionary:notification.userInfo];
        
        // glucose
        [self.glucoseLabel setText:[values valueForKey:@"Glucose"]];
        
        // status
        switch ([[values valueForKey:@"BgmState"] intValue])
        {
            case BGM_WAITING_FOR_STRIP_STATE:
                [self.bgmStateLabel setText:@"Insert Strip"];
                break;
            case BGM_STRIP_INSERTED_STATE:
                [self.bgmStateLabel setText:@"Waiting for sample"];
                break;
            case BGM_MEASURE_STATE:
                [self.bgmStateLabel setText:@"Measuring..."];
                break;
            case BGM_FINISH_STATE:
                [self.bgmStateLabel setText:@"Complete"];
                break;
            case BGM_FAILED_STATE:
                [self.bgmStateLabel setText:@"Failed"];
                break;
            case BGM_IDLE_STATE:
            default:
                [self.bgmStateLabel setText:@"Idle"];
                break;
        }
    }
}

- (void) glucoseDisconnect:(NSNotification *) notification
{
    if ([[notification name] isEqualToString:@"BleDisconnected"]) {
        // glucose
        [self.glucoseLabel setText:@"-"];
        // status
        [self.bgmStateLabel setText:@"-"];
    }
}

- (void)dealloc {
    // Must remove observer; otherwise the Notification Center will
    // continue to try and send notification objects to the deallocated
    // object
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
