//
//  StartViewController.m
//  BLE Solution
//
//  Created by Mark F Rodriguez on 7/26/16.
//  Copyright © 2016 Mark F Rodriguez. All rights reserved.
//

#import "StartViewController.h"

@interface StartViewController ()

@end

@implementation StartViewController
@synthesize discovery;
@synthesize _state;

#define STATES_DICT_FILTER(x)       @#x, [NSNumber numberWithInt:x],

- (id)initWithCoder:(NSCoder *)aDecoder {
    if (self = [super initWithCoder:aDecoder]) {
        // State names for debugging
        self.stateNames = [NSDictionary dictionaryWithObjectsAndKeys:STATES_LIST(STATES_DICT_FILTER) nil];
        self._state = -1;
    }
    
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // set version information
    NSString *version = [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleShortVersionString"];
    NSString *build = [[NSBundle mainBundle] objectForInfoDictionaryKey:(NSString *)kCFBundleVersionKey];
    [self.versionLabel setText: [NSString stringWithFormat:@"Version %@ (%@)", version, build]];
    
    // set stream flag
    self.isFirstDataStream = YES;
    
    // Do any additional setup after loading the view, typically from a nib.
    self.discovery = [LeDiscovery sharedInstance];
    discovery.discoveryDelegate = self;
    discovery.notificationDelegate = self;
    
    self.state = STATE_DISCONNECTED;
    
    // setup UI labels and notifity other tabs to clear out any shown data
    [self configureButton];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)discoveryDidRefresh {
    NSString *status = [NSString stringWithFormat:@"Not connected"];
    
    if (STATE_STOP == self.state) {
        NSLog(@"ignoring refresh, stopped");
        return;
    }
    
    NSLog(@"discoveryDidRefresh perpiherals: %@", discovery.foundPeripherals);
    
    for (CBPeripheral *peripheral in discovery.foundPeripherals) {
        if (NSOrderedSame == [peripheral.name compare:@"Renesas HC Demo"]) {
            switch (peripheral.state) {
                case CBPeripheralStateConnected:
                    status = @"Waiting for data...";
                    self.state = STATE_CONNECTED;
                    self.peripheral = peripheral;
                    break;
                    
                case CBPeripheralStateDisconnected:
                    //status = @"Connecting...";
                    self.state = STATE_DISCONNECTED;
                    [discovery connectPeripheral:peripheral];
                    break;
                    
                case CBPeripheralStateConnecting:
                    self.state = STATE_CONNECTING;
                    status = @"Connecting...";
                    break;
                    
                default:
                    NSLog(@"unknown peripheral.state %ld", (long)peripheral.state);
                    break;
            }
        }
        else {
            NSLog(@"uknown peripheral");
        }
    }
    
    NSLog(@"status: %@", status);
    [self.labelStatus setText:status];
    [self configureButton];
}

- (void)notification:(NSString *)identifier values:(NSDictionary *)values {
    if (self.isFirstDataStream) {
        // now really connected AND streaming data
        [self.labelStatus setText:@"Connected"];
        self.state = STATE_STREAM;
        NSLog(@"Streaming data...");
        self.isFirstDataStream = NO;
    }
    
    if ([identifier caseInsensitiveCompare:HC_ACTIVITY_SENSOR] == NSOrderedSame) {
        // broadcast values via Notification Center
        [[NSNotificationCenter defaultCenter] postNotificationName:@"ActivityDataReceived" object:self userInfo:values];
    }
    
    if ([identifier caseInsensitiveCompare:HC_HEALTHCARE_SENSOR] == NSOrderedSame) {
        // broadcast values via Notification Center
        [[NSNotificationCenter defaultCenter] postNotificationName:@"HealthDataReceived" object:self userInfo:values];
    }
}

- (void)configureButton {
    if (self.peripheral.state == CBPeripheralStateConnected) {
        [self.buttonConnect setTitle:@"Disconnect" forState:UIControlStateNormal];
    } else if (self.peripheral.state == CBPeripheralStateConnecting) {
        // do nothing
    } else if (self.peripheral.state == CBPeripheralStateDisconnecting) {
        // do nothing
    } else {
        // reset stream status
        self.isFirstDataStream = YES;
        // diable LED buttons
        [self.buttonConnect setTitle:@"Connect" forState:UIControlStateNormal];
        // notifiy other tabs about disconnection
        [[NSNotificationCenter defaultCenter] postNotificationName:@"BleDisconnected" object:self userInfo:nil];
    }
}

- (void)disconnected {
    if (STATE_STOP != self.state) {
        self.state = STATE_DISCONNECTED;
    }
    
    self.peripheral = nil;
    [self.labelStatus setText:@"Disconnected"];
    [self configureButton];
}

- (void)discoveryStatePoweredOff {
    NSLog(@"powered off");
}

- (void)errorOccured:(NSString *)error {
    NSLog(@"%@: setting peripheral to nil", error);
    
    if (self.peripheral) {
        [discovery disconnect];
        [discovery stopScanning];
    }
    
    self.peripheral = nil;
    [self.labelStatus setText:error];
    [self configureButton];
}

- (IBAction)buttonEmailPressed:(id)sender {
    // tear down any BLE connections if active
    if (self.peripheral.state == CBPeripheralStateConnected) {
        NSLog(@"Stopping BLE activity");
        self.state = STATE_STOP;
        [discovery disconnect];
        [discovery stopScanning];
        [self.labelStatus setText:@"Disconnecting..."];
    }
    else {
        if (self.state == STATE_DISCOVER) {
            // looking to cancel current scan
            NSLog(@"Canceling BLE activity");
            self.state = STATE_STOP;
            if (self.peripheral.state == CBPeripheralStateConnected) {
                [discovery disconnect];
            }
            [discovery stopScanning];
            [self.labelStatus setText:@"Disconnected"];
            [self configureButton];
        }
    }
    
    // Seque to email screen
    [self performSegueWithIdentifier:@"showEmail" sender:self];
}

- (IBAction)buttonConnectPressed:(id)sender {
    if (self.peripheral.state == CBPeripheralStateConnected) {
        NSLog(@"buttonPressed stop");
        self.state = STATE_STOP;
        [discovery disconnect];
        [discovery stopScanning];
        [self.labelStatus setText:@"Disconnecting..."];
    }
    else {
        if (self.state == STATE_DISCOVER) {
            // looking to cancel current scan
            NSLog(@"buttonPressed cancel");
            self.state = STATE_STOP;
            if (self.peripheral.state == CBPeripheralStateConnected) {
                [discovery disconnect];
            }
            [discovery stopScanning];
            [self.labelStatus setText:@"Disconnected"];
            [self configureButton];
        }
        else {
            NSLog(@"buttonPressed start");
            self.state = STATE_DISCOVER;
            [discovery startScanningForUUIDString:nil];
            [self.labelStatus setText:@"Scanning..."];
            [self.buttonConnect setTitle:@"Cancel" forState:UIControlStateNormal];
        }
    }
}

- (NSString *)stateToString:(enum BleState)state {
    NSString *result = [self.stateNames objectForKey:[NSNumber numberWithInt:state]];
    
    if (!result) {
        result = [NSString stringWithFormat:@"unknown ViewControllerState %d", (int)state];
    }
    
    return result;
}

- (void)setState:(enum BleState)state {
    if (state != _state) {
        NSLog(@"%@", [self stateToString:state]);
        _state = state;
    }
}

- (enum BleState)getState {
    return _state;
}


@end
