/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * pulse_adc_ssp.c
 *
 *  Created on: Jan 8, 2016
 *      Author: alejandro
 ****************************************************************************************
 */

#include <stdint.h>
#include "pulse_adc.h"
#include "pulse_ox_thread.h"

// mfr FIXME: - find out set values
#define PULSE_SETTLE_TIME_USEC 0
#define PULSE_IDLE_TIME_USEC 400

/**
 * Synergy/SSP specific implementation and resources
 */
const pmcu_resource_t pulse_adc_mcu_res =
{
     .timer_arm = &g_timer_2,
     .timer_settle = &g_timer_3,
     .adc = &g_adc,
     .event = &pulse_ox_app_event,
};

int hal_pulse_adc_notify_init (const pmcu_resource_t *c)
{
    ssp_err_t err;

    err = SSP_SUCCESS;

    return err;
}

int hal_pulse_adc_notify (const pmcu_resource_t *c, int event)
{
    int status;

    status = tx_event_flags_set (c->event, (ULONG)event, TX_OR);

    return status;
}

int hal_pulse_adc_wait_for_event (pulse_adc_sample_context_t *c, int *event, int *adc)
{
    int status;
    ULONG actual_flags;

    status = tx_event_flags_get(c->res->event, PULSE_R_EVENT_FLAG | PULSE_IR_EVENT_FLAG, TX_OR_CLEAR,
                                 &actual_flags, TX_WAIT_FOREVER);

    if (TX_SUCCESS == status)
    {
        // return the flags
        *event = (int)actual_flags;
        //
        adc[PULSE_LED_RED] = c->sample[PULSE_LED_RED].value;
        adc[PULSE_LED_IRED] = c->sample[PULSE_LED_IRED].value;
    }

    return status;
}

int hal_sample_adc_open (const pmcu_resource_t *c)
{
    ssp_err_t err;
    adc_ctrl_t *ctrl = c->adc->p_ctrl;
    adc_cfg_t const *cfg = c->adc->p_cfg;
    adc_api_t const *api = c->adc->p_api;

    adc_channel_cfg_t const *ch_cfg = c->adc->p_channel_cfg;

    err = api->open(ctrl, cfg);

    if (SSP_SUCCESS == err)
    {
        err = api->scanCfg (ctrl, ch_cfg);
    }

    return err;
}

int hal_sample_timer_open (const pmcu_resource_t *c)
{
    ssp_err_t err;
    timer_ctrl_t *ctrl = c->timer_arm->p_ctrl;
    timer_cfg_t const *cfg = c->timer_arm->p_cfg;
    timer_api_t const *api = c->timer_arm->p_api;

    // open the sample adc timer, it will auto start
    err = api->open (ctrl, cfg);

    return err;
}

int hal_sample_settle_timer_open (const pmcu_resource_t *c)
{
    ssp_err_t err;
    timer_ctrl_t *ctrl = c->timer_settle->p_ctrl;
    timer_cfg_t const *cfg = c->timer_settle->p_cfg;
    timer_api_t const *api = c->timer_settle->p_api;
    // open but do not start yet
    err = api->open (ctrl, cfg);

    return err;
}

void hal_adc_settle_timer_start (const pmcu_resource_t *c)
{
    c->timer_settle->p_api->periodSet (c->timer_settle->p_ctrl, c->timer_settle->p_cfg->period, c->timer_settle->p_cfg->unit);
    c->timer_settle->p_api->start (c->timer_settle->p_ctrl);
}

void hal_adc_idle_timer_start (const pmcu_resource_t *c)
{
    c->timer_settle->p_api->periodSet (c->timer_settle->p_ctrl, PULSE_IDLE_TIME_USEC, TIMER_UNIT_PERIOD_USEC);
    c->timer_settle->p_api->start (c->timer_settle->p_ctrl);
}

void hal_adc_convert_start (const pmcu_resource_t *c)
{
    c->adc->p_api->scanStart(c->adc->p_ctrl);
}

int hal_pulse_adc_sample_read (pulse_adc_sample_context_t *c, pulse_adc_sample_led_t *sample)
{
    ssp_err_t err;
    adc_api_t const *api = c->res->adc->p_api;
    adc_ctrl_t *ctrl = c->res->adc->p_ctrl;
    adc_data_size_t value;

    err = api->read (ctrl, sample->channel, &value);

    if (SSP_SUCCESS == err)
    {
        sample->value = value;
    }
    else
    {
        // TODO:
        // handle error condition
    }

    return err;
}

/**
 * GPT Timer channel 2 ISR callback
 */
void timer_2_callback (timer_callback_args_t * p_args)
{
    //pulse_bias_toggle (); // just for debugging purposes
    pulse_adc_sample_start_start (&pulse_adc_sample_context);
}

/**
 * GPT Timer channel 3 ISR callback
 */
void timer_3_callback (timer_callback_args_t * p_args)
{
    pulse_adc_sample_arm_handle (&pulse_adc_sample_context);
}

/**
 * ADC ISR callback
 */
void adc_callback (adc_callback_args_t * p_args)
{
    //pulse_bias_toggle ();
    pulse_adc_sample_handle (&pulse_adc_sample_context);
}
